from typing import Optional
from pydantic import BaseModel, Field

from .enums import ReviewSortBy


class BaseRequest(BaseModel):
    q: str = Field(..., description="The query to search for")
    gl: Optional[str] = Field(
        None, description="The country to search in, e.g. us, uk, ca, au, etc."
    )
    location: Optional[str] = Field(
        None, description="The location to search in, e.g. San Francisco, CA, USA"
    )
    hl: Optional[str] = Field(
        None, description="The language to search in, e.g. en, es, fr, de, etc."
    )
    page: Optional[str] = Field(
        "1",
        pattern=r"^[1-9]\d*$",
        description="The page number to return, first page is 1 (integer value as string)",
    )


class SearchRequest(BaseRequest):
    tbs: Optional[str] = Field(
        None, description="The time period to search in, e.g. d, w, m, y"
    )
    num: str = Field(
        "10",
        pattern=r"^([1-9]|[1-9]\d|100)$",
        description="The number of results to return, max is 100 (integer value as string)",
    )


class AutocorrectRequest(BaseRequest):
    autocorrect: Optional[str] = Field(
        "true",
        pattern=r"^(true|false)$",
        description="Automatically correct (boolean value as string: 'true' or 'false')",
    )


class MapsRequest(BaseModel):
    q: str = Field(..., description="The query to search for")
    ll: Optional[str] = Field(None, description="The GPS position & zoom level")
    placeId: Optional[str] = Field(None, description="The place ID to search in")
    cid: Optional[str] = Field(None, description="The CID to search in")
    gl: Optional[str] = Field(
        None, description="The country to search in, e.g. us, uk, ca, au, etc."
    )
    hl: Optional[str] = Field(
        None, description="The language to search in, e.g. en, es, fr, de, etc."
    )
    page: Optional[str] = Field(
        "1",
        pattern=r"^[1-9]\d*$",
        description="The page number to return, first page is 1 (integer value as string)",
    )


class ReviewsRequest(BaseModel):
    fid: str = Field(..., description="The FID")
    cid: Optional[str] = Field(None, description="The CID to search in")
    placeId: Optional[str] = Field(None, description="The place ID to search in")
    sortBy: Optional[str] = Field(
        "mostRelevant",
        pattern=r"^(mostRelevant|newest|highestRating|lowestRating)$",
        description="The sort order to use (enum value as string: 'mostRelevant', 'newest', 'highestRating', 'lowestRating')",
    )
    topicId: Optional[str] = Field(None, description="The topic ID to search in")
    nextPageToken: Optional[str] = Field(None, description="The next page token to use")
    gl: Optional[str] = Field(
        None, description="The country to search in, e.g. us, uk, ca, au, etc."
    )
    hl: Optional[str] = Field(
        None, description="The language to search in, e.g. en, es, fr, de, etc."
    )


class ShoppingRequest(BaseRequest):
    autocorrect: Optional[str] = Field(
        "true",
        pattern=r"^(true|false)$",
        description="Automatically correct (boolean value as string: 'true' or 'false')",
    )
    num: str = Field(
        "10",
        pattern=r"^([1-9]|[1-9]\d|100)$",
        description="The number of results to return, max is 100 (integer value as string)",
    )


class LensRequest(BaseModel):
    url: str = Field(..., description="The url to search")
    gl: Optional[str] = Field(
        None, description="The country to search in, e.g. us, uk, ca, au, etc."
    )
    hl: Optional[str] = Field(
        None, description="The language to search in, e.g. en, es, fr, de, etc."
    )


class PatentsRequest(BaseModel):
    q: str = Field(..., description="The query to search for")
    num: str = Field(
        "10",
        pattern=r"^([1-9]|[1-9]\d|100)$",
        description="The number of results to return, max is 100 (integer value as string)",
    )
    page: Optional[str] = Field(
        "1",
        pattern=r"^[1-9]\d*$",
        description="The page number to return, first page is 1 (integer value as string)",
    )


class WebpageRequest(BaseModel):
    url: str = Field(..., description="The url to scrape")
    includeMarkdown: Optional[str] = Field(
        "false",
        pattern=r"^(true|false)$",
        description="Include markdown in the response (boolean value as string: 'true' or 'false')",
    )



from typing import Any, List, Sequence
from mcp.server import Server
from mcp.server.stdio import stdio_server
from mcp.types import Tool, TextContent, ImageContent, EmbeddedResource
from dotenv import load_dotenv
import json

from .core import google, scape, SERPER_API_KEY
from .enums import SerperTools
from .schemas import (
    SearchRequest,
    MapsRequest,
    ReviewsRequest,
    ShoppingRequest,
    LensRequest,
    AutocorrectRequest,
    PatentsRequest,
    WebpageRequest
)

load_dotenv()

server = Server("Serper")

google_request_map = {
    SerperTools.GOOGLE_SEARCH: SearchRequest,
    SerperTools.GOOGLE_SEARCH_IMAGES: SearchRequest,
    SerperTools.GOOGLE_SEARCH_VIDEOS: SearchRequest,
    SerperTools.GOOGLE_SEARCH_PLACES: AutocorrectRequest,
    SerperTools.GOOGLE_SEARCH_MAPS: MapsRequest,
    SerperTools.GOOGLE_SEARCH_REVIEWS: ReviewsRequest,
    SerperTools.GOOGLE_SEARCH_NEWS: SearchRequest,
    SerperTools.GOOGLE_SEARCH_SHOPPING: ShoppingRequest,
    SerperTools.GOOGLE_SEARCH_LENS: LensRequest,
    SerperTools.GOOGLE_SEARCH_SCHOLAR: AutocorrectRequest,
    SerperTools.GOOGLE_SEARCH_PATENTS: PatentsRequest,
    SerperTools.GOOGLE_SEARCH_AUTOCOMPLETE: AutocorrectRequest,
}


@server.list_tools()
async def list_tools() -> List[Tool]:
    tools = []

    for k, v in google_request_map.items():
        tools.append(
            Tool(
                name=k.value,
                description="Search Google for results",
                inputSchema=v.model_json_schema(),
            ),
        )

    tools.append(Tool(
        name=SerperTools.WEBPAGE_SCRAPE,
        description="Scrape webpage by url",
        inputSchema=WebpageRequest.model_json_schema(),
    ))

    return tools

@server.call_tool()
async def call_tool(name: str, arguments: dict[str, Any]) -> Sequence[TextContent | ImageContent | EmbeddedResource]:
    if not SERPER_API_KEY:
        return [TextContent(text=f"SERPER_API_KEY is empty!", type="text")]

    try:
        if name == SerperTools.WEBPAGE_SCRAPE.value:
            request = WebpageRequest(**arguments)
            result = await scape(request)
            return [TextContent(text=json.dumps(result, indent=2), type="text")]

        if not SerperTools.has_value(name):
            raise ValueError(f"Tool {name} not found")

        tool = SerperTools(name)
        request = google_request_map[tool](**arguments)
        result = await google(tool, request)
        return [TextContent(text=json.dumps(result, indent=2), type="text")]
    except Exception as e:
        return [TextContent(text=f"Error: {str(e)}", type="text")]


async def main():
    options = server.create_initialization_options()
    async with stdio_server() as (read_stream, write_stream):
        await server.run(read_stream, write_stream, options)



import asyncio
import argparse
from . import server


def main():
    # parser = argparse.ArgumentParser()
    # parser.add_argument("--q", type=str, help="The query to search for")
    # args = parser.parse_args()
    asyncio.run(server.main())


__all__ = ["main"]



from enum import StrEnum


class SerperTools(StrEnum):
    GOOGLE_SEARCH = "google_search"
    GOOGLE_SEARCH_IMAGES = "google_search_images"
    GOOGLE_SEARCH_VIDEOS = "google_search_videos"
    GOOGLE_SEARCH_PLACES = "google_search_places"
    GOOGLE_SEARCH_MAPS = "google_search_maps"
    GOOGLE_SEARCH_REVIEWS = "google_search_reviews"
    GOOGLE_SEARCH_NEWS = "google_search_news"
    GOOGLE_SEARCH_SHOPPING = "google_search_shopping"
    GOOGLE_SEARCH_LENS = "google_search_lens"
    GOOGLE_SEARCH_SCHOLAR = "google_search_scholar"
    GOOGLE_SEARCH_PATENTS = "google_search_patents"
    GOOGLE_SEARCH_AUTOCOMPLETE = "google_search_autocomplete"
    WEBPAGE_SCRAPE = "webpage_scrape"

    @classmethod
    def has_value(cls, value: str) -> bool:
        return value in cls._value2member_map_


class ReviewSortBy(StrEnum):
    mostRelevant = "mostRelevant"
    newest = "newest"
    highestRating = "highestRating"
    lowestRating = "lowestRating"



import os
import ssl
from typing import Dict, Any
import certifi
import aiohttp
from pydantic import BaseModel
from .enums import SerperTools
from .schemas import WebpageRequest

SERPER_API_KEY = str.strip(os.getenv("SERPER_API_KEY", ""))
AIOHTTP_TIMEOUT = int(os.getenv("AIOHTTP_TIMEOUT", "15"))


async def google(tool: SerperTools, request: BaseModel) -> Dict[str, Any]:
    uri_path = tool.value.split("_")[-1]
    url = f"https://google.serper.dev/{uri_path}"
    return await fetch_json(url, request)


async def scape(request: WebpageRequest) -> Dict[str, Any]:
    url = "https://scrape.serper.dev"
    return await fetch_json(url, request)


async def fetch_json(url: str, request: BaseModel) -> Dict[str, Any]:
    payload = request.model_dump(exclude_none=True)
    headers = {
        'X-API-KEY': SERPER_API_KEY,
        'Content-Type': 'application/json'
    }

    ssl_context = ssl.create_default_context(cafile=certifi.where())
    connector = aiohttp.TCPConnector(ssl=ssl_context)

    timeout = aiohttp.ClientTimeout(total=AIOHTTP_TIMEOUT)
    async with aiohttp.ClientSession(connector=connector, timeout=timeout) as session:
        async with session.post(url, headers=headers, json=payload) as response:
            response.raise_for_status()
            return await response.json()
